# cdp/libs/sim/simulatorFuncs.py                               DMQ 12/03/03
"""Functions for a generic simulator.  Use this module as a pattern to
integrate your own simulator.
"""
import os
from os import listdir
from os.path import isdir, isfile, basename, join

def parseMods(modlist):
    """Convert a compact model list into a simple, fully-expanded
model list, suitable for input to a dumb simulator: one model per
line, with full pathnames.  Use only the first model of any with
identical filenames.
"""
    mlist = []; plist = []  # model and path lists
    for row in modlist:     # process rows top to bottom
        row = row.strip()   # strip off space on ends
        rs = row.split(';') # separate section name
        if len(rs) == 1:    # no section name
            row = rs[0]; section = ''
        elif len(rs) == 2:  # normal section name
            row = rs[0]; section = rs[1]
        else:
            print "***WARNING*** Bad Syntax:\n", rs
#        print row,'***',section
        
        if isdir(row):
            rlist = listdir(row)
            for mf in rlist:  # assume all items are simple model files
                rmf = join(row, mf)
                if not isfile(rmf):
                    print "***WARNING*** Not a file:\n", rmf
                    break
                # more tests here (section name, etc.)
                if mf not in mlist: # skip later models with same name
                    mlist.append(mf)
                    plist.append(rmf + '   ' + section)
                        # use form required by your simulator.
        elif isfile(row):
            mf = basename(row)
            if mf not in mlist:   # ditto for simple files
                mlist.append(mf)
                plist.append(row + '   ' + section)
        else:
            print "*** WARNING *** Not a file or dir:\n", "'" + row + "'"
            
    return plist

def getSampleModlist():
    # Uncomment one:
    base = os.environ['HOME'] + "/cad/models"  # Local files
#    base = "//sahuaro/macquigg/cad/models"     # Remote files
    modelsetup = (
    base + "/cs65/spectre/custom/pnps.scs;slow \n" +
    base + "/gpdk/nmos.scs  \n" +
    base + "/gpdk/deadlink/ \n" +
    base + "/cs65/spectre/allnom/" )
    return modelsetup

