#!/usr/bin/env python
# ../CDP/Statefiles/Bag/bag02.py                         DMQ 5/19/04
# Organize statefiles as nested "bags" of parameters.
# Load and save in a neat format for external editing.

TAB = 3; SP = ' '*(TAB-1); LINE = 0; LMAX = 15
TABMARKS = (('#'+SP)+('.'+SP)*4)*3 + '\n'

class Bag:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        
    def load(infile):
        strng = infile.read()
        exec( 'bag = Bag(\n' + strng + ')' )
        return bag
    load = staticmethod(load)
    
    def save(self, outfile, indent=''):
        global LINE
        for k,v in self.__dict__.items():
            LINE += 1
            if isinstance( v, Bag ):
                if LINE > LMAX:
                    outfile.write( TABMARKS ); LINE = 0
                outfile.write( indent + k + '=Bag(\n' )
                v.save(outfile, indent + ' '*TAB )
                outfile.write( indent + '),\n' )
            else:
                if type(v) is not str:
                    v = str(v)
                else:
                    v = "'%s'" % v
                outfile.write( indent + k + ' = ' + v + ',\n' )

if __name__ == '__main__':
    # Input can be cut-and-paste from an existing statefile:
    settings = Bag(
    #   .   .   .   .   #   .   .   .   .   
    logging=Bag(
    #   .   .   .   .   #   .   .   .   .   
        errors=Bag(
            level = 3,
        ),
    ),
    #   .   .   .   .   #   .   .   .   .   
    network=Bag(
        ip = '10.20.30.40',
        useSSL = True,
        ports = [1234, 2345, 3456]
    ),
    modelsetup = [
    "/cs65/spectre/custom/pnps.scs;slow \n",
    "/gpdk/nmos.scs  \n",
    "/gpdk/deadlink/ \n",
    "/cs65/spectre/allnom/ \n"
    ]
    )

    # Read and write settings
    print settings.network.ip #=>
    # 10.20.30.40
    settings.network.useSSL = False
    print settings.network.useSSL # =>
    # False
    print settings.network.__dict__ # =>
    # {'ip': '10.20.30.40', 'useSSL': False, 'Indent': '', 'port': 1234}


    # Save to a statefile
    outfile = file('statefile', 'w')
    ## import sys
    ## outfile = sys.stdout  # for test output to console
    settings.save(outfile)
    outfile.close()

    # Read a big statefile
    infile = file('bignest.bag', 'r')
    parms = Bag.load(infile)
    infile.close()

    # Write a big statefile
    outfile = file('bignest1.bag','w')
    parms.save(outfile)
    outfile.close()

    # Save a pickle
    import cPickle
    pkfile = file('bignest.pk','w')
    pk = cPickle.Pickler(pkfile)
    pk.dump(parms)
    pkfile.close()

    # Load a pickle
    pkfile = file('bignest.pk','r')
    pk = cPickle.Unpickler(pkfile)
    parm2 = pk.load()

    raw_input('Press Enter to exit')  # Windows trick.
